==> financial-front-2.0.0.tar.gz: financial-front-2.0.0/app/models/account_title.rb <==
# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# 勘定科目
class AccountTitle < ActiveRecord::Base
  belongs_to :receivable, :foreign_key => 'receivable_id',
                          :class_name => 'AccountTitle'
  belongs_to :payable, :foreign_key => 'payable_id',
                       :class_name => 'AccountTitle'
  
  # 集計先
  has_many :account_title_trees, :foreign_key => 'parent_id'
  has_many :children, :through => :account_title_trees

  VAT_NAMES = {10 => '税込5%', 20 => '外税5%', 30 => '非課税', 40 => '―'}
  KIND_NAMES = {
    1 => "売掛金・未収入金", 2 => "買掛金・未払金", 3 => "仮払消費税", 
    0 => "その他"}

  validates_presence_of :account_code
  validates_presence_of :suppl_code
  validates_uniqueness_of :suppl_code, :scope => :account_code
  validates_presence_of :main_name
  validates_presence_of :name
  validates_presence_of :kana_name

  before_save :normalize_string
  
  def normalize_string
    self.main_name = normalize_kc main_name
    self.name = normalize_kc name
    self.kana_name = normalize_kc(kana_name).gsub(/[ 　\t\r\n・]/, '')
    self.description = normalize description, :c
  end

  class << self
    # (購買)請求書で追加可能な科目一覧を得る
    def addable
      find :all,
         :conditions => '(end_day IS NULL) AND (payable_id IS NOT NULL) AND (aggregate_flag IS FALSE) AND kind = 0',
         :order => 'account_code, suppl_code'
    end

    # 販売請求書での追加可能な科目一覧
    def sales_addable
      find :all,
         :conditions => '(end_day IS NULL) AND (receivable_id IS NOT NULL) AND (aggregate_flag IS FALSE)',
         :order => 'account_code, suppl_code'
    end

    # 売掛金・未収入金科目の一覧
    def receivable_candidates
      AccountTitle.find :all,
               :conditions => '(end_day IS NULL) AND kind = 1',
               :order => 'account_code, suppl_code' 
    end

    # 買掛金・未払金科目の一覧
    def payable_candidates
      AccountTitle.find :all,
               :conditions => '(end_day IS NULL) AND kind = 2',
               :order => 'account_code, suppl_code'
    end
  
    # 送金手数料
    def bank_charge
      GlobalSetting.first.bank_charge_account_title
    end
  
    def currency_gain
      GlobalSetting.first.currency_gain_account_title
    end
  

    # 仮払消費税
    def vat_payment
      GlobalSetting.first.vat_payment_account_title
    end


    def vat_ary(ary = nil)
      a = VAT_NAMES.to_a.sort {|x, y| x[0] <=> y[0]}
      r = []
      a.each {|k, v|
        r << [v, k] if !ary || ary.include?(k)
      }
      return r
    end

  end # class << self

  # @override
  def validate
    if sign != 1 && sign != -1
      errors.add 'sign', '内部エラー: 符号は+1または-1に限る'
    end
  end

  def name2
    return "#{account_code} #{name}"
  end
  
  # 自分自身と祖先は禁止
  def self_and_ancestors
    ret = last = [self.id]
    while !last.empty?
      last = AccountTitleTree.find(:all,
                                   :conditions => ['child_id IN (?)', last]
             ).collect {|r| r.parent_id}
      ret += last
    end
    return ret
  end

  # 直接の子供と子孫が被ったら警告
  # 直接の子供ではない子孫を返す
  def descendant
    ret = []
    last = AccountTitleTree.find(:all,
               :conditions => ['parent_id = ?', self.id]).collect {|r| r.child_id}
    while !last.empty?
      last = AccountTitleTree.find(:all,
               :conditions => ['parent_id IN (?)', last]).collect {|r| r.child_id}
      ret += last
    end
    return ret
  end

  # 仕訳での消費税コード
  def get_real_vat_code(vat_or_zero)
    if vat_or_zero.to_i != 0
      vat_or_zero
    else
      if vat_code != 0
        vat_code
      else
        10 # TODO:決め打ち
      end
    end
  end

end

==> financial-front-2.99.0.tar.gz: financial-front-2.99.0/app/models/account_title.rb <==
# -*- coding: utf-8 -*-

# Netsphere Financial Front / ネットスフィア経理フロント
# Copyright (C) 2007-2009,2011 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


require "application_helper"


# 勘定科目
class AccountTitle < ActiveRecord::Base
  belongs_to :receivable, :foreign_key => 'receivable_id',
                          :class_name => 'AccountTitle'
  belongs_to :payable, :foreign_key => 'payable_id',
                       :class_name => 'AccountTitle'
  
  # 集計先
  has_many :account_title_trees, :foreign_key => 'parent_id'
  has_many :children, :through => :account_title_trees

  # for compat.
  VAT_NAMES = {10 => '税込5%', 20 => '外税5%', 30 => '非課税', 40 => '―'}

  KIND_NAMES = {
    1 => "売掛金・未収入金", 2 => "買掛金・未払金", 3 => "仮払消費税", 
    0 => "その他"}

  validates_presence_of :account_code
  validates_presence_of :suppl_code
  validates_uniqueness_of :suppl_code, :scope => :account_code
  validates_presence_of :main_name
  validates_presence_of :name
  validates_presence_of :kana_name

  validate :check_fields

  before_save :normalize_string
  
  def normalize_string
    self.main_name = normalize_kc main_name
    self.name = normalize_kc name
    self.kana_name = normalize_kc(kana_name).gsub(/[ 　\t\r\n・]/, '')
    self.description = normalize description, :c
  end


  class << self
    # (購買)請求書で追加可能な科目一覧を得る
    def addable
      find :all,
         :conditions => '(end_day IS NULL) AND (enable_invoice IS TRUE) AND (aggregate_flag IS FALSE) AND kind = 0',
         :order => 'account_code, suppl_code'
    end

    # 販売請求書での追加可能な科目一覧
    def sales_addable
      find :all,
         :conditions => '(end_day IS NULL) AND (enable_si_invoice IS TRUE) AND (aggregate_flag IS FALSE)',
         :order => 'account_code, suppl_code'
    end


    # 売掛金・未収入金科目の一覧
    def receivable_candidates
      AccountTitle.find :all,
               :conditions => '(end_day IS NULL) AND kind = 1',
               :order => 'account_code, suppl_code' 
    end

    # 買掛金・未払金科目の一覧
    def payable_candidates
      AccountTitle.find :all,
               :conditions => '(end_day IS NULL) AND kind = 2',
               :order => 'account_code, suppl_code'
    end
  
    # 送金手数料
    def bank_charge
      GlobalSetting.first.bank_charge_account_title
    end
  
    def currency_gain
      GlobalSetting.first.currency_gain_account_title
    end
  

    # 仮払消費税
    def vat_payment
      GlobalSetting.first.vat_payment_account_title
    end


    def vat_ary(ary = nil)
      a = VAT_NAMES.to_a.sort {|x, y| x[0] <=> y[0]}
      r = []
      a.each {|k, v|
        r << [v, k] if !ary || ary.include?(k)
      }
      return r
    end

  end # class << self


  # for validate
  def check_fields
    if sign != 1 && sign != -1
      errors.add 'sign', '内部エラー: 符号は+1または-1に限る'
    end
  end

  def name2
    return "#{account_code} #{name}"
  end
  
  # 自分自身と祖先は禁止
  def self_and_ancestors
    ret = last = [self.id]
    while !last.empty?
      last = AccountTitleTree.find(:all,
                                   :conditions => ['child_id IN (?)', last]
             ).collect {|r| r.parent_id}
      ret += last
    end
    return ret
  end

  # 直接の子供と子孫が被ったら警告
  # 直接の子供ではない子孫を返す
  def descendant
    ret = []
    last = AccountTitleTree.find(:all,
               :conditions => ['parent_id = ?', self.id]).collect {|r| r.child_id}
    while !last.empty?
      last = AccountTitleTree.find(:all,
               :conditions => ['parent_id IN (?)', last]).collect {|r| r.child_id}
      ret += last
    end
    return ret
  end


  # @param vat_id_or_zero  -1       ... 科目に従う
  #                        0 or nil ... nil
  #                        >= 1     ... vat id
  # @param country 2文字コード ex) "JP"
  def get_real_vat vat_id_or_zero, country
    raise TypeError if !country.is_a?(String) || country.length != 2

    if vat_id_or_zero.to_i >= 1
      Vat.find vat_id_or_zero
    elsif vat_id_or_zero.to_i == 0
      nil
    else
      Vat.find :first, 
             :conditions => ["tax_side = ? AND tax_group = ? AND country = ?",
                             self.tax_side, self.default_tax_type, country]
    end
  end

end
